/* summarysidebar.cc
 * Copyright (C) 2001 The University of Newcastle upon Tyne
 *
 * Written by Paul Coates (Paul.Coates@ncl.ac.uk)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#include <gtk/gtk.h>
#include <string.h>
#include <stdio.h>
#include "summarysidebar.h"
#include "session.h"
#include "data.h"
#include "monitor.h"
#include "gui.h"
#include "main.h"
#include "smachine.xpm"
#include "srouter.xpm"
#include "scloud.xpm"
#include "sswitch.xpm"
#include "ssubnet.xpm"

SummarySideBar::SummarySideBar(Main *m)
{
	main = m;

	GtkWidget *label183;
	GtkWidget *hbox3;
	GtkWidget *scrolledwindow1;
	GtkWidget *label4;
	GtkWidget *label5;

  vbox2 = gtk_vbox_new (FALSE, 2);
  gtk_widget_show (vbox2);
  gtk_container_set_border_width (GTK_CONTAINER (vbox2), 2);

  label183 = gtk_label_new ("Session Summary");
  gtk_widget_show (label183);
  gtk_box_pack_start (GTK_BOX (vbox2), label183, FALSE, FALSE, 0);
  gtk_misc_set_alignment (GTK_MISC (label183), 0, 0.5);

  hbox3 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox3);
  gtk_box_pack_start (GTK_BOX (vbox2), hbox3, FALSE, TRUE, 0);
  gtk_widget_set_usize (hbox3, -2, 120);

  scrolledwindow1 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (scrolledwindow1);
  gtk_box_pack_start (GTK_BOX (hbox3), scrolledwindow1, TRUE, TRUE, 0);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow1),
		GTK_POLICY_NEVER, GTK_POLICY_NEVER);

  clist1 = gtk_clist_new (2);
  gtk_widget_show (clist1);
  gtk_container_add (GTK_CONTAINER (scrolledwindow1), clist1);
  gtk_clist_set_column_width (GTK_CLIST (clist1), 0, 74);
  gtk_clist_set_column_width (GTK_CLIST (clist1), 1, 24);
  gtk_clist_column_titles_show (GTK_CLIST (clist1));

  label4 = gtk_label_new ("Type");
  gtk_widget_show (label4);
  gtk_clist_set_column_widget (GTK_CLIST (clist1), 0, label4);
  gtk_misc_set_alignment (GTK_MISC (label4), 0, 0.5);

  label5 = gtk_label_new ("Found");
  gtk_widget_show (label5);
  gtk_clist_set_column_widget (GTK_CLIST (clist1), 1, label5);
  gtk_misc_set_alignment (GTK_MISC (label5), 0, 0.5);
}

SummarySideBar::~SummarySideBar()
{
}

void SummarySideBar::init()
{
	GtkWidget *window;
	gchar *data[2];

	gtk_clist_clear(GTK_CLIST(clist1));

	data[0] = strdup("");
	data[1] = strdup("0");
	gtk_clist_append(GTK_CLIST(clist1), data);
	gtk_clist_append(GTK_CLIST(clist1), data);
	gtk_clist_append(GTK_CLIST(clist1), data);
	gtk_clist_append(GTK_CLIST(clist1), data);
	gtk_clist_append(GTK_CLIST(clist1), data);
	delete data[0]; delete data[1];

	window = main->gui->monitor->mainwindow;
	clist_pixtext(window, GTK_CLIST(clist1), 0, 0, srouter_xpm, "Routers");
	clist_pixtext(window, GTK_CLIST(clist1), 1, 0, sswitch_xpm, "Switches");
	clist_pixtext(window, GTK_CLIST(clist1), 2, 0, smachine_xpm, "Hosts");
	clist_pixtext(window, GTK_CLIST(clist1), 3, 0, scloud_xpm, "Networks");
	clist_pixtext(window, GTK_CLIST(clist1), 4, 0, ssubnet_xpm, "Subnets");
}

void SummarySideBar::update()
{
	char buf[64];
	Link *ln;
	Device *d;
	int ns=0, nr=0;
	for (ln=main->session->data->devices.get_head(); ln!=NULL; ln=ln->next)
	{
		d = (Device*) ln->item;
		if (d->type=='s') ns++; else
		if (d->type=='r') nr++;
	}

	if (main->session->data!=NULL)
	{
		sprintf(buf, "%d", nr);
		gtk_clist_set_text(GTK_CLIST(clist1), 0, 1, buf);
		sprintf(buf, "%d", ns);
		gtk_clist_set_text(GTK_CLIST(clist1), 1, 1, buf);
		sprintf(buf, "%d", main->session->data->machines.size());
		gtk_clist_set_text(GTK_CLIST(clist1), 2, 1, buf);
		sprintf(buf, "%d", main->session->data->networks.size());
		gtk_clist_set_text(GTK_CLIST(clist1), 3, 1, buf);
		sprintf(buf, "%d", main->session->data->subnets.size());
		gtk_clist_set_text(GTK_CLIST(clist1), 4, 1, buf);
	}
}

