/* tasklist.cc
 * Copyright (C) 2001 The University of Newcastle upon Tyne
 *
 * Written by Paul Coates (Paul.Coates@ncl.ac.uk)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#include <glade/glade-xml.h>
#include <gtk/gtk.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <stdio.h>
#include "session.h"
#include "tasklist.h"
#include "main.h"
#include "misc.h"
#include "gui.h"
#include "scheduler.h"
#include "task.h"
#include "licon.xpm"
#include "sup.xpm"
#include "sdel.xpm"
#include "sdown.xpm"
#include "supall.xpm"
#include "sdelall.xpm"
#include "sdownall.xpm"

static gint delete_event(GtkWidget *widget, GdkEvent *event, gpointer data)
{ ((TaskList*)data)->hide(); return (TRUE); }
static void close_cb(GtkWidget *w, gpointer data)
{ ((TaskList*)data)->hide(); }

TaskList::TaskList(Main *m)
{
	GladeXML *gxml;

	main = m;

	gchar *fname = user_file("nomad.glade");
	gxml = glade_xml_new (fname, "tasklist");
	if (!gxml)
	{
		g_warning("Failed to load file nomad.glade");
		return;
	}

	window = glade_xml_get_widget (gxml, "tasklist");
	button7 = glade_xml_get_widget (gxml, "button7");
	button8 = glade_xml_get_widget (gxml, "button8");
	button9 = glade_xml_get_widget (gxml, "button9");
	button10 = glade_xml_get_widget (gxml, "button10");
	button11 = glade_xml_get_widget (gxml, "button11");
	button12 = glade_xml_get_widget (gxml, "button12");
	button13 = glade_xml_get_widget (gxml, "button13");
	clist5 = glade_xml_get_widget (gxml, "clist5");
	gtk_object_unref (GTK_OBJECT (gxml));
	g_free(fname);

	gtk_widget_set_usize (window, 420, 280);
	gtk_widget_realize (window);
	window_icon(window, licon_xpm);
	gdk_window_set_icon_name(window->window, "Task List");
	gtk_signal_connect (GTK_OBJECT (window), "delete_event",
		GTK_SIGNAL_FUNC(delete_event), this);
	gtk_signal_connect(GTK_OBJECT(window), "destroy",
		GTK_SIGNAL_FUNC(delete_event), this);

	pix[0] = create_pixmap(window, sdownall_xpm);
	pix[1] = create_pixmap(window, sdown_xpm);
	pix[2] = create_pixmap(window, sup_xpm);
	pix[3] = create_pixmap(window, supall_xpm);
	pix[4] = create_pixmap(window, sdel_xpm);
	pix[5] = create_pixmap(window, sdelall_xpm);

	gtk_container_add (GTK_CONTAINER (button8), pix[0]);
	gtk_container_add (GTK_CONTAINER (button9), pix[1]);
	gtk_container_add (GTK_CONTAINER (button10), pix[2]);
	gtk_container_add (GTK_CONTAINER (button11), pix[3]);
	gtk_container_add (GTK_CONTAINER (button13), pix[4]);
	gtk_container_add (GTK_CONTAINER (button12), pix[5]);

	gtk_signal_connect (GTK_OBJECT (button7), "pressed",
		GTK_SIGNAL_FUNC (close_cb), this);
	gtk_signal_connect (GTK_OBJECT (button8), "pressed",
		GTK_SIGNAL_FUNC (close_cb), this);
	gtk_signal_connect (GTK_OBJECT (button9), "pressed",
		GTK_SIGNAL_FUNC (close_cb), this);
	gtk_signal_connect (GTK_OBJECT (button10), "pressed",
		GTK_SIGNAL_FUNC (close_cb), this);
	gtk_signal_connect (GTK_OBJECT (button11), "pressed",
		GTK_SIGNAL_FUNC (close_cb), this);
	gtk_signal_connect (GTK_OBJECT (button12), "pressed",
		GTK_SIGNAL_FUNC (close_cb), this);
	gtk_signal_connect (GTK_OBJECT (button13), "pressed",
		GTK_SIGNAL_FUNC (close_cb), this);

//	gtk_signal_connect(GTK_OBJECT(clist5), "select_row",
//		GTK_SIGNAL_FUNC(interface_clist_cb), this);
//	gtk_signal_connect(GTK_OBJECT(clist5), "unselect_row",
//		GTK_SIGNAL_FUNC(interface_clist_cb2), this);
}

TaskList::~TaskList()
{
}

void TaskList::show()
{
	gtk_widget_show(window);
	gdk_window_raise(window->window);
}

void TaskList::hide()
{
	gtk_widget_hide(window);
}

void TaskList::update()
{
	Link *ln;
	Task *t;
	gchar *data[3];
	char buf[64];
	int lvl;
	gtk_clist_freeze(GTK_CLIST(clist5));
	gtk_clist_clear(GTK_CLIST(clist5));
	for (lvl=0; lvl<=6; lvl++)
	{
		for (ln=main->scheduler->tasks.get_head(); ln!=NULL; ln=ln->next)
		{
			t = (Task*) ln->item;
			if (t->priority==lvl)
			{
				sprintf(buf, "%d", lvl);
				data[0] = strdup(buf);
				data[1] = strdup(buf);
				data[2] = strdup(t->cmnd);
				gtk_clist_append(GTK_CLIST(clist5), data);
				delete data[0]; delete data[1]; delete data[2];
			}
		}
	}
	gtk_clist_thaw(GTK_CLIST(clist5));
}

