/* main.cc
 * Copyright (C) 2001 The University of Newcastle upon Tyne
 *
 * Written by Paul Coates (Paul.Coates@ncl.ac.uk)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <stdio.h>
#include <pwd.h>
#include "main.h"
#include "header.h"

int main(int argc, char *argv[])
{
	Main m;

	if (argc!=2)
	{
		printf("usage:%s <session name>\n", argv[0]);
		return 1;
	}

	strcpy(m.session, argv[1]);

	m.run();
	m.quit();

	return 0;
}

void Main::run()
{
	char buf[256];
	struct passwd *pwd;
	Process *p;
	Link *ln;
	char *md;
	Task *t;
	int i, flag;

	md = getenv("NOMAD_DIR");
	if (md!=NULL) { strcpy(datadir, md); } else
	{ pwd = getpwuid(getuid());
		sprintf(datadir, "%s/%s", pwd->pw_dir, DATA_DIR);
	}

	for (i=1; i<=NUM_PROCESSES; i++)
	{
		p = new Process();
		p->create();
		processes.tag(p);
	}

	stat_tm = time(NULL);

	while(1)
	{
		// check for processes ending and set appropriate flags
		for (ln=processes.get_head(); ln!=NULL; ln=ln->next)
		{
			p = (Process*) ln->item;
			if (p->get(buf))
			{
				check_incoming_task(&stat_tasks, p);
				p->tid = 0;
			}
		}

		process_tasks(&stat_tasks);

		flag = 0;
		for (ln=stat_tasks.get_head(); ln!=NULL; ln=ln->next)
		{
			t = (Task*) ln->item;
			if (t->status == 0) flag = 1;
		}
		if (flag == 0)	// all processes returned
		{
			stat_tm = time(NULL);
			tasks_from_devices(&stat_tasks, "nomad-traffic");
		}

//		status();
		usleep(2000000);
	}
}

// check the task list for the task that is being run by the returning process
void Main::check_incoming_task(List *lst, Process *p)
{
	Link *ln2;
	Task *t;

	for (ln2=lst->get_head(); ln2!=NULL; ln2=ln2->next)
	{
		t = (Task*) ln2->item;
		if (t->pid == p->pid)
		{
			t->pid = 0;
			t->status = 1;
		}
	}
}

// create a task list from the devices file
void Main::tasks_from_devices(List *lst, char *str)
{
	char buf[256];
	char buf2[256];
	char buf3[256];
	char buf4[256];
	char line[256];
	FILE *f;
	Task *t;
	Link *ln;
	int i, count;

	// delete any existing tasks
	for (ln=lst->get_head(); ln!=NULL; ln=ln->next)
	{
		t = (Task*) ln->item;
		delete t;
	}
	lst->empty();

	// create a new list
	count = 0;
	sprintf(buf, "%s/sessions/%s/devices", datadir, session);
	f = fopen(buf, "r");
	if (f)
	{
		while(fgets(line, 256, f) != NULL)
		{
			if (sscanf(line, "%[^,],%[^,],%[^\n]\n", buf2, buf3, buf4)==3)
			{
				t = new Task();
				sprintf(t->cmnd, "%s %s %s", str, buf3, buf4);
				lst->tag(t);
				count++;
			}
		}
		fclose(f);
	}

	i = 0;
	for (ln=lst->get_head(); ln!=NULL; ln=ln->next)
	{
		t = (Task*)ln->item;
		t->ts = (int)((300*i)/count);
		i++;
	}
}

// take in a task list and assign what you can to any free processes
void Main::process_tasks(List *lst)
{
	Link *ln, *ln2;
	Process *p;
	Task *t;
	time_t now_tm = time(NULL);

	for (ln=processes.get_head(); ln!=NULL; ln=ln->next)
	{
		p = (Process*)ln->item;
		if (p->tid==0)
		{
			ln2=lst->get_head();
			while(ln2!=NULL)
			{
				t = (Task*) ln2->item;
				if ((t->status==0) && (t->pid==0) && (now_tm >= stat_tm+t->ts))
				{
					t->pid = p->pid;
					p->tid = 1;
					strcpy(p->cmnd, t->cmnd);
					p->send(t->cmnd);
					ln2=NULL;
				}else
				{
					ln2=ln2->next;
				}
			}
		}
	}
}

void Main::status()
{
	Link *ln;
	Process *p;
	Task *t;
	int i;

	i=1;
	for (ln=processes.get_head(); ln!=NULL; ln=ln->next)
	{
		p = (Process*)ln->item;
		printf("%2d %6d ", i, p->pid);
		if (p->tid!=0)
		{
			printf("%s", p->cmnd);
		}
		printf("\n");
		i++;
	}

	for (ln=stat_tasks.get_head(); ln!=NULL; ln=ln->next)
	{
		t = (Task*)ln->item;
		printf("%4d %d %8d %s\n", t->pid, t->status, t->ts, t->cmnd);
	}
}

void Main::quit()
{
	Link *ln;

	for (ln=processes.get_head(); ln!=NULL; ln=ln->next)
		((Process*)ln->item)->send("quit");
}

