/* device.cc
 * Copyright (C) 2001 The University of Newcastle upon Tyne
 *
 * Written by Paul Coates (Paul.Coates@ncl.ac.uk)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#include <string.h>
#include <stdlib.h>
#include "device.h"
#include "interface.h"
#include "filter.h"
#include "misc.h"

Device::Device()
{
	empty();
}

Device::~Device()
{
	empty();
}

void Device::empty()
{
	Interface *in;
	MACAddress *m;
	Link *ln;
	int i;

	x = rand()%4000;
	y = rand()%3000;
	w = h = 192;
	selected = '0';
	show = '1';
	type = 'u';
	fix = '0';
	visited = '0';
	find = '0';

	pingname.empty();
	pingname2.empty();
	strcpy(name, "");
	last_seen.empty();
	strcpy(location, "");
	strcpy(community, "");
	strcpy(community2, "");
	strcpy(descr, "");
	strcpy(comment, "");
	strcpy(contact, "");
	status = '0';
	group = 'b';
	grab_top = '0';
	grab_stats = '0';
	for (ln=interfaces.get_head(); ln!=NULL; ln=ln->next)
	{
		in = (Interface*) ln->item;
		if (in!=NULL) delete in;
	}
	interfaces.empty();
	for (ln=loopbacks.get_head(); ln!=NULL; ln=ln->next)
	{
		m = (MACAddress*) ln->item;
		if (m!=NULL) delete m;
	}
	loopbacks.empty();
	merged = 0;
	being_grabbed = '0';
	find = '0';
	tag = 0;
}

void Device::savexml(xmlNodePtr xmlnode)
{
	xmlNodePtr newxml;
	xmlNodePtr cur;
	char buf[4];
	Link *ln;
	Interface *in;
	MACAddress *m;

	newxml = xmlNewChild(xmlnode, NULL, (xmlChar*)"Device", NULL);
	if (!pingname.zero())
		xmlSetProp(newxml, (xmlChar*)"pingname", (xmlChar*)pingname.getstring());
	if (strcmp(name, "")!=0)
		xmlSetProp(newxml, (xmlChar*)"name", (xmlChar*)name);
	if (!last_seen.zero())
		xmlSetProp(newxml, (xmlChar*)"lastseen", (xmlChar*)last_seen.getstring());
	if (strcmp(location, "")!=0)
		xmlSetProp(newxml, (xmlChar*)"location", (xmlChar*)location);
	if (strcmp(descr, "")!=0)
		xmlSetProp(newxml, (xmlChar*)"description", (xmlChar*)descr);
	if (strcmp(comment, "")!=0)
		xmlSetProp(newxml, (xmlChar*)"comment", (xmlChar*)descr);
	if (strcmp(contact, "")!=0)
		xmlSetProp(newxml, (xmlChar*)"contact", (xmlChar*)contact);
	if (strcmp(community, "")!=0)
		xmlSetProp(newxml, (xmlChar*)"community", (xmlChar*)community);
//	sprintf(buf, "%c", registered);
//	xmlSetProp(newxml, (xmlChar*)"registered", (xmlChar*)buf);
//	sprintf(buf, "%c", matrix);
//	xmlSetProp(newxml, (xmlChar*)"matrix", (xmlChar*)buf);
	sprintf(buf, "%c", grab_top);
	xmlSetProp(newxml, (xmlChar*)"grab_topology", (xmlChar*)buf);
	sprintf(buf, "%c", grab_stats);
	xmlSetProp(newxml, (xmlChar*)"grab_stats", (xmlChar*)buf);
	sprintf(buf, "%c", type);
	xmlSetProp(newxml, (xmlChar*)"type", (xmlChar*)buf);

	for (ln=loopbacks.get_head(); ln!=NULL; ln=ln->next)
	{
		cur = xmlNewChild(newxml, NULL, (xmlChar*)"Loopback", NULL);
		m = (MACAddress*) ln->item;
		if (m!=NULL)
			xmlSetProp(cur, (xmlChar*)"mac", (xmlChar*)m->getstring());
	}

	for (ln=interfaces.get_head(); ln!=NULL; ln=ln->next)
	{
		in = (Interface*) ln->item;
		if (in!=NULL)
			in->savexml(newxml);
	}
}

void Device::loadxml(xmlNodePtr xmlnode)
{
	xmlChar *buf;
	Interface *in;
	MACAddress *m;

	empty();

	if (!xmlnode || !xmlnode->name || strcmp((char*)xmlnode->name, "Device")!=0)
		return;
	buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"pingname");
	if (buf!=NULL) { pingname.setstring((char*)buf); free(buf); }
	buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"name");
	if (buf!=NULL) { strcpy(name, (char*)buf); free(buf); }
	buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"lastseen");
	if (buf!=NULL) { last_seen.setstring((char*)buf); free(buf); }
	buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"location");
	if (buf!=NULL) { strcpy(location, (char*)buf); free(buf); }
	buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"description");
	if (buf!=NULL) { strcpy(descr, (char*)buf); free(buf); }
	buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"comment");
	if (buf!=NULL) { strcpy(comment, (char*)buf); free(buf); }
	buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"contact");
	if (buf!=NULL) { strcpy(contact, (char*)buf); free(buf); }
	buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"community");
	if (buf!=NULL) { strcpy(community, (char*)buf); free(buf); }
//	buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"registered");
//	if (buf!=NULL) { registered = buf[0]; free(buf); }
//	buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"matrix");
//	if (buf!=NULL) { matrix = buf[0]; free(buf); }
	buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"grab_topology");
	if (buf!=NULL) { grab_top = buf[0]; free(buf); }
	buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"grab_stats");
	if (buf!=NULL) { grab_stats = buf[0]; free(buf); }
	buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"type");
	if (buf!=NULL) { type = buf[0]; free(buf); }

	xmlnode = xmlnode->childs;
	while (xmlnode != NULL)
	{
		if (xmlnode->name!=NULL)
		{
			if (strcmp((char*)xmlnode->name, "Interface")==0)
			{
				in = new Interface();
				in->loadxml(xmlnode);
				in->device = this;
				interfaces.tag(in);
			}else
			if (strcmp((char*)xmlnode->name, "Loopback")==0)
			{
				m = new MACAddress();
				buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"mac");
				if (buf!=NULL) { m->setstring((char*)buf); free(buf); }
				loopbacks.tag(m);
			}
		}
		xmlnode = xmlnode->next;
	}
}

int compare_device_names(Link *a, Link *b)
{
	Device *r1 = (Device*) a->item;
	Device *r2 = (Device*) b->item;
	return (strcmp(r1->name, r2->name));
}

int compare_device_ips(Link *a, Link *b)
{
	Device *r1 = (Device*) a->item;
	Device *r2 = (Device*) b->item;
	return (r1->pingname.compare(r2->pingname));
}

void Device::match(char *str)
{
	find = '0';
	if (strlen(str)==0) return;
	if ((strlen(name)>0) && (lower_contains(name, str))) find = '1';
}

void Device::matchfilter(Filter *f)
{
	Link *ln, *ln2;
	IPAddress *tip;
	Interface *in;

	if (f->default_action=='d')
		show='0';
	else
		show='1';

	if (f->default_action=='d')
	{
		if (f->match(pingname)!=0) { show='1'; return; }
	}else
	{
		if (f->match(pingname)==0) { show='0'; return; }				
	}
	for (ln=interfaces.get_head(); ln!=NULL; ln=ln->next)
	{
		in = (Interface*) ln->item;
		for (ln2=in->ip_addresses.get_head(); ln2!=NULL; ln2=ln2->next)
		{
			tip = (IPAddress*) ln2->item;
			if (f->default_action=='d')
			{
				if (f->match(*tip)!=0) { show='1'; return; }
			}else
			{
				if (f->match(*tip)==0) { show='0'; return; }				
			}
		}
	}
}

