#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <pwd.h>
#include "misc.h"
#include "header.h"

int contains(char *str, char *substr)
{
	int a, b, i, j;
	char buf[256];

	a = strlen(str);
	b = strlen(substr);

	for (i=0; i<a-b+1; i++)
	{
		for (j=0; j<b; j++)
		{
			buf[j] = str[j+i];
		}
		buf[b] = '\0';
		if ((strcmp(buf, substr))==0) return 1;
	}

	return 0;
}

int lower_contains(char *str, char *substr)
{
	int a, b, i;
	char buf1[256];
	char buf2[256];

	strcpy(buf1, str);
	strcpy(buf2, substr);

	a = strlen(buf1);
	b = strlen(buf1);

	for (i=0; i<a; i++)
		if ((buf1[i]>='A') && (buf1[i]<='Z')) buf1[i] = buf1[i] - 'A' + 'a';
	for (i=0; i<b; i++)
		if ((buf2[i]>='A') && (buf2[i]<='Z')) buf2[i] = buf2[i] - 'A' + 'a';
	buf1[a] = '\0'; buf2[b] = '\0';

	return contains(buf1, buf2);
}

int starts_with(char *str, char *bigstr)
{
	char buf[256];
	strcpy(buf, bigstr);
	buf[strlen(str)] = '\0';
	return (strcmp(buf, str)==0);
}

/////////////////////////////////////////////////////
// We use our own read/write functions so we don't
// have to worry about endian issues across platforms
/////////////////////////////////////////////////////

void write_int32(FILE *f, int val)
{
	fputc((val & 0xff000000)>>24, f);
	fputc((val & 0xff0000)>>16, f);
	fputc((val & 0xff00)>>8, f);
	fputc((val & 0xff), f);
}

void write_ulong32(FILE *f, unsigned long val)
{
	fputc((val & 0xff000000)>>24, f);
	fputc((val & 0xff0000)>>16, f);
	fputc((val & 0xff00)>>8, f);
	fputc((val & 0xff), f);
}

int read_int32(FILE *f)
{
	int a, b, c, d;
	a = fgetc(f);
	b = fgetc(f);
	c = fgetc(f);
	d = fgetc(f);
	return (((((a*256)+b)*256)+c)*256)+d;
}

unsigned long read_ulong32(FILE *f)
{
	int a, b, c, d;
	a = fgetc(f);
	b = fgetc(f);
	c = fgetc(f);
	d = fgetc(f);
	return (((((a*256)+b)*256)+c)*256)+d;
}

///////////////////////////////////////////////////////////
// we also have 64 bit versions of the read/write
// functions for future expansion
///////////////////////////////////////////////////////////

void write_int64(FILE *f, int val)
{
	fputc(0, f);
	fputc(0, f);
	fputc(0, f);
	fputc(0, f);
	fputc((val & 0xff000000)>>24, f);
	fputc((val & 0xff0000)>>16, f);
	fputc((val & 0xff00)>>8, f);
	fputc((val & 0xff), f);
}

void write_ulong64(FILE *f, unsigned long val)
{
	fputc(0, f);
	fputc(0, f);
	fputc(0, f);
	fputc(0, f);
	fputc((val & 0xff000000)>>24, f);
	fputc((val & 0xff0000)>>16, f);
	fputc((val & 0xff00)>>8, f);
	fputc((val & 0xff), f);
}

int read_int64(FILE *f)
{
	int a, b, c, d;
	// we throw away the first 4 bytes
//	fseek(f, 4L, SEEK_CUR);
	a = fgetc(f);
	a = fgetc(f);
	a = fgetc(f);
	a = fgetc(f);
	a = fgetc(f);
	b = fgetc(f);
	c = fgetc(f);
	d = fgetc(f);
	return (((((a*256)+b)*256)+c)*256)+d;
}

unsigned long read_ulong64(FILE *f)
{
	int a, b, c, d;
	// we throw away the first 4 bytes
//	fseek(f, 4L, SEEK_CUR);
	a = fgetc(f);
	a = fgetc(f);
	a = fgetc(f);
	a = fgetc(f);
	a = fgetc(f);
	b = fgetc(f);
	c = fgetc(f);
	d = fgetc(f);
	return (((((a*256)+b)*256)+c)*256)+d;
}

int send_email(char *address, char *subject, char *message)
{
	FILE *f;

	f=popen(MAILCMD, "w");
	if (!f) return 0;

	if (f)
	{
		fprintf(f, "HELO\n");
		fprintf(f, "MAIL FROM:<%s>\n", SENDER);
		fprintf(f, "RCPT TO:<%s>\n", address);
		fprintf(f, "DATA\n");
		fprintf(f, "From: %s\n",SENDER);
		fprintf(f, "Subject: %s\n", subject);
		fprintf(f, "Sender: %s\n",SENDER);
		fprintf(f, "To: %s\n", address);
		fprintf(f, "\n\n");
		fprintf(f, "%s\n", message);
		fprintf(f, "\n\n.\n");
		fprintf(f, "QUIT\n");
		pclose(f);
	}
	return 1;
}

// Searches a number of possible locations for the file
// returns full path to file or NULL if not found

char *user_file(char *fname)
{
	struct stat statbuf;
	struct passwd *pwd;
	char *name[6];
	char *file = NULL;
	char buf[256];
	FILE *f;
	int i;

	pwd = getpwuid(getuid());
	sprintf(buf, "%s/%s/%s", pwd->pw_dir, DATA_DIR, fname);
	name[0] = strdup(buf);
	name[1] = strdup(fname);
	sprintf(buf, "../%s", fname);		name[2] = strdup(buf);
	sprintf(buf, "ui/%s", fname);		name[3] = strdup(buf);
	sprintf(buf, "../ui/%s", fname);		name[4] = strdup(buf);
	sprintf(buf, "%s/%s", SHARE_DIR, fname);		name[5] = strdup(buf);

	for (i=0; i<6; i++)
	{
//printf("CHECKING FOR FILE %s\n", name[i]);
		if (stat(name[i], &statbuf)==0)
		{
//printf("*** FOUND %s ***\n", name[i]);
			if (file) free(file);
			file = strdup(name[i]);
			i=6;
		}
	}

	for (i=0; i<6; i++)
	{
		free(name[i]);
	}

	return file;
}

