#include <string.h>
#include <stdlib.h>
#include "network.h"
#include "device.h"
#include "subnet.h"
#include "interface.h"
#include "machine.h"
#include "filter.h"

Network::Network()
{
	empty();
}

Network::~Network()
{
	empty();
}

void Network::empty()
{
	Interface *in;
	Machine *m;
	MACAddress *mac;
	Subnet *s;
	Link *ln;

	x = rand()%4000;
	y = rand()%3000;
	w = h = 96;
	selected = '0';
	show = '1';
	type = 'n';
	fix = '0';
	visited = '0';
	find = '0';

	view = 's';
	group = 'b';
	position = 'a';
//	for (ln=machines.get_head(); ln!=NULL; ln=ln->next)
//	{ m = (Machine*) ln->item; if (m!=NULL) delete m; }
//	for (ln=subnets.get_head(); ln!=NULL; ln=ln->next)
//	{ s = (Subnet*) ln->item; if (s!=NULL) delete s; }
//	for (ln=devices.get_head(); ln!=NULL; ln=ln->next)
//	{ mac = (MACAddress*) ln->item; if (mac!=NULL) delete mac; }
	interfaces.empty();
	machines.empty();
	subnets.empty();
	devices.empty();
}

void Network::savexml(xmlNodePtr xmlnode)
{
	xmlNodePtr newxml;
	xmlNodePtr cur;
	Interface *in;
	MACAddress *mac;
	Machine *m;
	Subnet *sn;
	Device *d;
	Link *ln;
	char buf[256];

	newxml = xmlNewChild(xmlnode, NULL, (xmlChar*)"Network", NULL);

	for (ln=interfaces.get_head(); ln!=NULL; ln=ln->next)
	{
		cur = xmlNewChild(newxml, NULL, (xmlChar*)"Iface", NULL);
		in = (Interface*) ln->item;
		if (in!=NULL)
		{
			d = (Device*) in->device;
			if (d!=NULL)
			{
				xmlSetProp(cur, (xmlChar*)"router", (xmlChar*)d->name);
				sprintf(buf, "%d", in->index);
				xmlSetProp(cur, (xmlChar*)"index", (xmlChar*)buf);
			}
		}
	}

	for (ln=machines.get_head(); ln!=NULL; ln=ln->next)
	{
		m = (Machine*) ln->item;
		if (m!=NULL)
			m->savexml(newxml);
	}

	for (ln=subnets.get_head(); ln!=NULL; ln=ln->next)
	{
		sn = (Subnet*) ln->item;
		if (sn!=NULL)
			sn->savexml(newxml);
	}

	for (ln=devices.get_head(); ln!=NULL; ln=ln->next)
	{
		cur = xmlNewChild(newxml, NULL, (xmlChar*)"Device", NULL);
		mac = (MACAddress*) ln->item;
		if (mac!=NULL)
			xmlSetProp(cur, (xmlChar*)"mac", (xmlChar*)mac->getstring());
	}
}

void Network::loadxml(xmlNodePtr xmlnode, List *rtr, List *mach, List *sns)
{
	xmlChar *buf;
	Link *ln, *ln2;
	Subnet sn, *sn2;
	Machine *m;
	Interface *in;
	MACAddress *mac;
	Device *d;
	char tname[65];
	int index;

	// check this is a router node
	if (!xmlnode || !xmlnode->name || strcmp((char*)xmlnode->name, "Network")!=0)
		return;

	interfaces.empty();
	subnets.empty();
	machines.empty();
	devices.empty();

	xmlnode = xmlnode->childs;
	while (xmlnode != NULL)
	{
		if (xmlnode->name!=NULL)
		{
			if (strcmp((char*)xmlnode->name, "Iface")==0)
			{
				buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"router");
				if (buf!=NULL) { strcpy(tname, (char*)buf); free(buf); }
				buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"index");
				if (buf!=NULL) { sscanf((char*)buf, "%d", &index); free(buf); }

				for (ln=rtr->get_head(); ln!=NULL; ln=ln->next)
				{
					d = (Device*) ln->item;
					if (d!=NULL)
					{
						if (strcmp(d->name, tname)==0)
						{
							for (ln2=d->interfaces.get_head(); ln2!=NULL; ln2=ln2->next)
							{
								in = (Interface*) ln2->item;
								if (in!=NULL)
								{
									if (in->index == index)
									{
										in->network = this;
										interfaces.tag(in);
									}
								}
							}
						}
					}
				}
			}else
			if (strcmp((char*)xmlnode->name, "Machine")==0)
			{
				m = new Machine();
				m->loadxml(xmlnode);
				m->network = this;
				machines.tag(m);
				mach->tag(m);
			}else
			if (strcmp((char*)xmlnode->name, "Device")==0)
			{
				mac = new MACAddress();
				buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"mac");
				if (buf!=NULL) { mac->setstring((char*)buf); free(buf); }
				devices.tag(m);
			}else
			if (strcmp((char*)xmlnode->name, "Subnet")==0)
			{
				sn.empty();
				sn.loadxml(xmlnode);
				for (ln=sns->get_head(); ln!=NULL; ln=ln->next)
				{
					sn2 = (Subnet*) ln->item;
					if ((sn2->ip.compare(sn.ip)==0) && (sn2->mask == sn.mask))
					{
						subnets.tag(sn2);
					}
				}
			}
		}
		xmlnode = xmlnode->next;
	}
}

void Network::matchfilter(Filter *f)
{
	Link *ln, *ln2;
	IPAddress *tip;
	Interface *in;
	Machine *m;

	if (f->default_action=='d')
		show='0';
	else
		show='1';

	for (ln=interfaces.get_head(); ln!=NULL; ln=ln->next)
	{
		in = (Interface*) ln->item;
		for (ln2=in->ip_addresses.get_head(); ln2!=NULL; ln2=ln2->next)
		{
			tip = (IPAddress*) ln2->item;
			if (f->default_action=='d')
			{
				if (f->match(*tip)!=0) { show='1'; return; }
			}else
			{
				if (f->match(*tip)==0) { show='0'; return; }				
			}
		}
	}
	for (ln=machines.get_head(); ln!=NULL; ln=ln->next)
	{
		m = (Machine*) ln->item;
		if (f->default_action=='d')
		{
			if (f->match(m->ip)!=0) { show='1'; return; }
		}else
		{
			if (f->match(m->ip)==0) { show='0'; return; }				
		}
	}
	// could also check subnets but should be already covered
}

/*
void Network::matchfilter(Filter *f)
{
	
}
*/

