/* switchgroup.cc
 * Copyright (C) 2001 The University of Newcastle upon Tyne
 *
 * Written by Paul Coates (Paul.Coates@ncl.ac.uk)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#include <string.h>
#include <stdlib.h>
#include "interface.h"

SwitchGroup::SwitchGroup()
{
	empty();
}

SwitchGroup::~SwitchGroup()
{
	empty();
}

void SwitchGroup::empty()
{
	Link *ln;

	// don't delete the switches just the list
	switches.empty();
}

void SwitchGroup::savexml(xmlNodePtr xmlnode)
{
	xmlNodePtr newxml;
	xmlNodePtr cur;
	Interface *in;
	MACAddress *mac;
	Machine *m;
	Subnet *sn;
	Device *d;
	Link *ln;
	char buf[256];

	newxml = xmlNewChild(xmlnode, NULL, (xmlChar*)"SwitchGroup", NULL);

	for (ln=interfaces.get_head(); ln!=NULL; ln=ln->next)
	{
		cur = xmlNewChild(newxml, NULL, (xmlChar*)"Iface", NULL);
		in = (Interface*) ln->item;
		if (in!=NULL)
		{
			d = (Device*) in->device;
			if (d!=NULL)
			{
				xmlSetProp(cur, (xmlChar*)"router", (xmlChar*)d->name);
				sprintf(buf, "%d", in->index);
				xmlSetProp(cur, (xmlChar*)"index", (xmlChar*)buf);
			}
		}
	}

	for (ln=machines.get_head(); ln!=NULL; ln=ln->next)
	{
		m = (Machine*) ln->item;
		if (m!=NULL)
			m->savexml(newxml);
	}

	for (ln=subnets.get_head(); ln!=NULL; ln=ln->next)
	{
		sn = (Subnet*) ln->item;
		if (sn!=NULL)
			sn->savexml(newxml);
	}

	for (ln=devices.get_head(); ln!=NULL; ln=ln->next)
	{
		cur = xmlNewChild(newxml, NULL, (xmlChar*)"Device", NULL);
		mac = (MACAddress*) ln->item;
		if (mac!=NULL)
			xmlSetProp(cur, (xmlChar*)"mac", (xmlChar*)mac->getstring());
	}
}

void SwitchGroup::loadxml(xmlNodePtr xmlnode, List *rtr, List *mach, List *sns)
{
	xmlChar *buf;
	Link *ln, *ln2;
	Subnet sn, *sn2;
	Machine *m;
	Interface *in;
	MACAddress *mac;
	Device *d;
	char tname[65];
	int index;

	// check this is a router node
	if (!xmlnode || !xmlnode->name || strcmp((char*)xmlnode->name, "SwitchGroup")!=0)
		return;

	interfaces.empty();
	subnets.empty();
	machines.empty();
	devices.empty();

	xmlnode = xmlnode->childs;
	while (xmlnode != NULL)
	{
		if (xmlnode->name!=NULL)
		{
			if (strcmp((char*)xmlnode->name, "Iface")==0)
			{
				buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"router");
				if (buf!=NULL) { strcpy(tname, (char*)buf); free(buf); }
				buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"index");
				if (buf!=NULL) { sscanf((char*)buf, "%d", &index); free(buf); }

				for (ln=rtr->get_head(); ln!=NULL; ln=ln->next)
				{
					d = (Device*) ln->item;
					if (d!=NULL)
					{
						if (strcmp(d->name, tname)==0)
						{
							for (ln2=d->interfaces.get_head(); ln2!=NULL; ln2=ln2->next)
							{
								in = (Interface*) ln2->item;
								if (in!=NULL)
								{
									if (in->index == index)
									{
										in->network = this;
										interfaces.tag(in);
									}
								}
							}
						}
					}
				}
			}else
			if (strcmp((char*)xmlnode->name, "Machine")==0)
			{
				m = new Machine();
				m->loadxml(xmlnode);
				m->network = this;
				machines.tag(m);
				mach->tag(m);
			}else
			if (strcmp((char*)xmlnode->name, "Device")==0)
			{
				mac = new MACAddress();
				buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"mac");
				if (buf!=NULL) { mac->setstring((char*)buf); free(buf); }
				devices.tag(m);
			}else
			if (strcmp((char*)xmlnode->name, "Subnet")==0)
			{
				sn.empty();
				sn.loadxml(xmlnode);
				for (ln=sns->get_head(); ln!=NULL; ln=ln->next)
				{
					sn2 = (Subnet*) ln->item;
					if ((sn2->ip.compare(sn.ip)==0) && (sn2->mask == sn.mask))
					{
						subnets.tag(sn2);
					}
				}
			}
		}
		xmlnode = xmlnode->next;
	}
}

