#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include "viewitem.h"
#include "view.h"
#include "device.h"
#include "network.h"
#include "interface.h"
#include "data.h"

View::View()
{
	strcpy(name, "");
	empty();
}

View::~View()
{
	strcpy(name, "");
	empty();
}

void View::empty()
{
	Link *ln;
	ViewItem *in;

	for (ln=devices.get_head(); ln!=NULL; ln=ln->next)
	{
		in = (ViewItem*) ln->item;
		if (in!=NULL) delete in;
		ln->item = NULL;
	}
	devices.empty();
	for (ln = networks.get_head(); ln!=NULL; ln=ln->next)
	{
		in = (ViewItem*) ln->item;
		if (in!=NULL) delete in;
		ln->item = NULL;
	}
	networks.empty();

	labels = '0';
	grid = '0';
	snap = '0';
}

void View::savexml(xmlNodePtr xmlnode)
{
	xmlNodePtr newxml;
	char buf[32];
	ViewItem *in;
	Link *ln;

	newxml = xmlNewChild(xmlnode, NULL, (xmlChar*)"View", NULL);
	if (strcmp(name, "")!=0)
		xmlSetProp(newxml, (xmlChar*)"name", (xmlChar*)name);
//	sprintf(buf, "%d", labels);
//	xmlSetProp(newxml, (xmlChar*)"labels", (xmlChar*)buf);
//	sprintf(buf, "%d", grid);
//	xmlSetProp(newxml, (xmlChar*)"grid", (xmlChar*)buf);
//	sprintf(buf, "%d", snap);
//	xmlSetProp(newxml, (xmlChar*)"snap", (xmlChar*)buf);
	for (ln=devices.get_head(); ln!=NULL; ln=ln->next)
	{
		in = (ViewItem*) ln->item;
		if (in!=NULL) in->savexml(newxml, 'd');
	}
	for (ln=networks.get_head(); ln!=NULL; ln=ln->next)
	{
		in = (ViewItem*) ln->item;
		if (in!=NULL) in->savexml(newxml, 'n');
	}
}

void View::loadxml(xmlNodePtr xmlnode)
{
	ViewItem *in;
	Device *r;
	Network *n;
	Link *ln;
	xmlChar *buf;

	strcpy(name, "");
	empty();
	if (!xmlnode || !xmlnode->name || strcmp((char*)xmlnode->name, "View")!=0)
		return;
	buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"name");
	if (buf!=NULL) { strcpy(name, (char*)buf); free(buf); }

	xmlnode = xmlnode->childs;
	while (xmlnode != NULL)
	{
		if (xmlnode->name!=NULL)
		{
			if (strcmp((char*)xmlnode->name, "Device")==0)
			{
				in = new ViewItem();
				in->loadxml(xmlnode);
				devices.tag(in);
			}else
			if (strcmp((char*)xmlnode->name, "Network")==0)
			{
				in = new ViewItem();	in->group = 's';	// default view
				in->loadxml(xmlnode);
				networks.tag(in);
			}
		}
		xmlnode = xmlnode->next;
	}
}

void View::copy_from_topology(Data *data)
{
	Link *ln, *ln2;
	Device *r;
	Network *n;
	Interface *in;
	ViewItem *itm;

	empty();

	for (ln = data->devices.get_head(); ln!=NULL; ln=ln->next)
	{
		r = (Device*) ln->item;
		itm = new ViewItem();
		itm->ip.set(r->pingname);
		itm->x = r->x;
		itm->y = r->y;
		itm->show = r->show;
		itm->group = r->group;
		devices.tag(itm);
	}

	for (ln = data->networks.get_head(); ln!=NULL; ln=ln->next)
	{
		n = (Network*) ln->item;
		ln2 = n->interfaces.get_head();
		if (ln2!=NULL)
		{
			in = (Interface*) ln2->item;
			itm = new ViewItem();
			itm->ip.set(in->device->pingname);
			itm->iface = in->index;
			itm->x = n->x;
			itm->y = n->y;
			itm->show = n->show;
			itm->group = n->view;
			networks.tag(itm);
		}
	}
}

void View::copy_to_topology(Data *data)
{
	Link *ln, *ln2, *ln3;
	ViewItem *itm;
	Device *r;
	Network *n;
	Interface *in;

	for (ln2 = data->devices.get_head(); ln2!=NULL; ln2=ln2->next)
	{
		r = (Device*) ln2->item;
		for (ln = devices.get_head(); ln!=NULL; ln=ln->next)
		{
			itm = (ViewItem*) ln->item;
			if (r->pingname.compare(itm->ip)==0)
			{
				r->x = itm->x;
				r->y = itm->y;
				r->show = itm->show;
				r->group = itm->group;
			}
		}
		for (ln = r->interfaces.get_head(); ln!=NULL; ln=ln->next)
		{
			in = (Interface*) ln->item;
			for (ln3 = networks.get_head(); ln3!=NULL; ln3=ln3->next)
			{
				itm = (ViewItem*) ln3->item;
				if ((r->pingname.compare(itm->ip)==0) && (itm->iface==in->index))
				{
					in->network->x = itm->x;
					in->network->y = itm->y;
					in->network->show = itm->show;
					in->network->view = itm->group;
					if (in->network->view=='f')
					{ in->network->w = 96*4; in->network->h = 96*4; }else
					if (in->network->view!='b')
					{ in->network->w = 96; in->network->h = 96; }
				}
			}
		}
	}
}

